import { createClient } from '@wix/sdk';
import { SiteSessionAuth } from '@wix/sdk/auth/site-session';

export const isBackofficeExtension = location.pathname.startsWith(
  '/_wix/extensions/backoffice'
);

function getCookieAsJson(name: string) {
  const cookies = document.cookie.split('; ');
  const cookie = cookies.find((row) => row.startsWith(`${name}=`));

  if (!cookie) {
    return null;
  }

  try {
    const jsonString = decodeURIComponent(cookie.split('=')[1] ?? '');
    return JSON.parse(jsonString);
  } catch (error) {
    // eslint-disable-next-line no-console
    console.error('Error parsing cookie JSON:', error);
    return null;
  }
}

export function getSessionClient(options: { clientId: string }) {
  return createClient({
    auth: SiteSessionAuth({
      tokens: getCookieAsJson('wixSession')?.tokens,
      ...options,
    }),
  });
}
