import { z } from 'zod';

export const baseDevCenterComponentSchema = z
  .object({
    compData: z.unknown(),
    compId: z.string(),
    compName: z.string().optional(),
    compType: z.string(),
  })
  .passthrough();

export type BaseDevCenterComponent = z.TypeOf<
  typeof baseDevCenterComponentSchema
>;

const devCenterWebhookSchema = baseDevCenterComponentSchema.merge(
  z.object({
    compData: z.object({
      webhook: z
        .object({
          callbackUrl: z.string().optional(),
        })
        .passthrough(),
    }),
    compType: z.literal('WEBHOOK'),
  })
);

export type DevCenterWebhook = z.TypeOf<typeof devCenterWebhookSchema>;

const devCenterBackofficeExtensionWidgetSchema =
  baseDevCenterComponentSchema.merge(
    z.object({
      compData: z.object({
        backOfficeExtensionWidget: z
          .object({
            iframeUrl: z.string().optional(),
          })
          .passthrough(),
      }),
      compType: z.literal('BACK_OFFICE_EXTENSION_WIDGET'),
    })
  );

export type DevCenterBackofficeExtensionWidget = z.TypeOf<
  typeof devCenterBackofficeExtensionWidgetSchema
>;

const devCenterBackofficeExtensionMenuPluginSchema =
  baseDevCenterComponentSchema.merge(
    z.object({
      compData: z.object({
        backOfficeExtensionMenuItem: z.unknown().optional(),
      }),
      compType: z.literal('BACK_OFFICE_EXTENSION_MENU_ITEM'),
    })
  );

export type DevCenterBackofficeExtensionMenuItem = z.TypeOf<
  typeof devCenterBackofficeExtensionMenuPluginSchema
>;

const devCenterBackofficeModalSchema = baseDevCenterComponentSchema.merge(
  z.object({
    compData: z.object({
      backOfficeModal: z
        .object({
          iframeUrl: z.string().optional(),
        })
        .passthrough(),
    }),
    compType: z.literal('BACK_OFFICE_MODAL'),
  })
);

export type DevCenterBackofficeModal = z.TypeOf<
  typeof devCenterBackofficeModalSchema
>;

const devCenterBackofficePageSchema = baseDevCenterComponentSchema.merge(
  z.object({
    compData: z.object({
      backOfficePage: z
        .object({
          iframeUrl: z.string().optional(),
        })
        .passthrough(),
    }),
    compType: z.literal('BACK_OFFICE_PAGE'),
  })
);

export type DevCenterBackofficePage = z.TypeOf<
  typeof devCenterBackofficePageSchema
>;

const devCenterEcomAdditionalFeesSchema = baseDevCenterComponentSchema.merge(
  z.object({
    compData: z.object({
      ecomAdditionalFees: z
        .object({
          deploymentUri: z.string().optional(),
        })
        .passthrough(),
    }),
    compType: z.literal('ECOM_ADDITIONAL_FEES'),
  })
);

export type DevCenterEcomAdditionalFees = z.TypeOf<
  typeof devCenterEcomAdditionalFeesSchema
>;

const devCenterEcomDiscountsTriggerSchema = baseDevCenterComponentSchema.merge(
  z.object({
    compData: z.object({
      ecomDiscountsTrigger: z
        .object({
          deploymentUri: z.string().optional(),
        })
        .passthrough(),
    }),
    compType: z.literal('ECOM_DISCOUNTS_TRIGGER'),
  })
);

export type DevCenterEcomDiscountsTrigger = z.TypeOf<
  typeof devCenterEcomDiscountsTriggerSchema
>;

const devCenterEcomPaymentSettingsSchema = baseDevCenterComponentSchema.merge(
  z.object({
    compData: z.object({
      ecomPaymentSettings: z
        .object({
          deploymentUri: z.string().optional(),
        })
        .passthrough(),
    }),
    compType: z.literal('ECOM_PAYMENT_SETTINGS'),
  })
);

export type DevCenterEcomPaymentSettings = z.TypeOf<
  typeof devCenterEcomPaymentSettingsSchema
>;

const devCenterEcomShippingRatesSchema = baseDevCenterComponentSchema.merge(
  z.object({
    compData: z.object({
      ecomShippingRates: z
        .object({
          deploymentUri: z.string().optional(),
        })
        .passthrough(),
    }),
    compType: z.literal('ECOM_SHIPPING_RATES'),
  })
);

export type DevCenterEcomShippingRates = z.TypeOf<
  typeof devCenterEcomShippingRatesSchema
>;

const devCenterEcomValidationsSchema = baseDevCenterComponentSchema.merge(
  z.object({
    compData: z.object({
      ecomValidations: z
        .object({
          deploymentUri: z.string().optional(),
        })
        .passthrough(),
    }),
    compType: z.literal('ECOM_VALIDATIONS'),
  })
);

export type DevCenterEcomValidations = z.TypeOf<
  typeof devCenterEcomValidationsSchema
>;

const devCenterEcomGiftCardsProviderSchema = baseDevCenterComponentSchema.merge(
  z.object({
    compData: z.object({
      giftCardsProvider: z
        .object({
          deploymentUri: z.string().optional(),
        })
        .passthrough(),
    }),
    compType: z.literal('GIFT_CARDS_PROVIDER'),
  })
);

export type DevCenterEcomGiftCardsProvider = z.TypeOf<
  typeof devCenterEcomGiftCardsProviderSchema
>;

export const devCenterComponentSchema = z.discriminatedUnion('compType', [
  devCenterBackofficePageSchema,
  devCenterBackofficeExtensionWidgetSchema,
  devCenterBackofficeExtensionMenuPluginSchema,
  devCenterBackofficeModalSchema,
  devCenterWebhookSchema,
  devCenterEcomAdditionalFeesSchema,
  devCenterEcomShippingRatesSchema,
  devCenterEcomDiscountsTriggerSchema,
  devCenterEcomValidationsSchema,
  devCenterEcomPaymentSettingsSchema,
  devCenterEcomGiftCardsProviderSchema,
]);

export type DevCenterComponent = z.infer<typeof devCenterComponentSchema>;

export interface DevCenterAppManifest {
  appId: string;
  components: BaseDevCenterComponent[];
}
