import {
  access,
  readFile as fsReadFile,
  mkdir,
  writeFile,
} from 'node:fs/promises';
import { EOL } from 'node:os';
import { dirname } from 'node:path';

function toJsonString(object: unknown, opts?: { spaces: number }) {
  return JSON.stringify(object, null, opts?.spaces).concat(EOL);
}

export async function writeJson(
  filePath: string,
  object: unknown,
  opts?: { spaces: number }
) {
  const str = toJsonString(object, opts);
  await outputDir(dirname(filePath));

  await writeFile(filePath, str, 'utf-8');
}

export async function readJson(file: string): Promise<unknown> {
  return JSON.parse(await fsReadFile(file, 'utf-8'));
}

export function pathExists(path: string) {
  return access(path)
    .then(() => true)
    .catch(() => false);
}

export async function outputDir(dir: string) {
  await mkdir(dir, { recursive: true });
}
