import type {
  DevCenterAppManifest,
  DevCenterBackofficeExtensionMenuItem,
  DevCenterBackofficeExtensionWidget,
  DevCenterBackofficeModal,
  DevCenterBackofficePage,
  DevCenterEcomAdditionalFees,
  DevCenterEcomDiscountsTrigger,
  DevCenterEcomGiftCardsProvider,
  DevCenterEcomPaymentSettings,
  DevCenterEcomShippingRates,
  DevCenterEcomValidations,
  DevCenterWebhook,
} from '../schemas.js';
import type { Model } from '../types.js';

export function generateAppManifest(model: Model) {
  const extensions = model.components
    .map((component) => component.manifest)
    .map((manifest) => {
      switch (manifest.compType) {
        case 'BACK_OFFICE_EXTENSION_MENU_ITEM':
          return manifest satisfies DevCenterBackofficeExtensionMenuItem;
        case 'BACK_OFFICE_EXTENSION_WIDGET':
          if (manifest.compData.backOfficeExtensionWidget.iframeUrl) {
            return manifest;
          }

          return {
            ...manifest,
            compData: {
              backOfficeExtensionWidget: {
                ...manifest.compData.backOfficeExtensionWidget,
                iframeUrl: `_wix/extensions/backoffice/${manifest.compId}`,
              },
            },
          } satisfies DevCenterBackofficeExtensionWidget;
        case 'BACK_OFFICE_MODAL':
          if (manifest.compData.backOfficeModal.iframeUrl) {
            return manifest;
          }

          return {
            ...manifest,
            compData: {
              backOfficeModal: {
                ...manifest.compData.backOfficeModal,
                iframeUrl: `_wix/extensions/backoffice/${manifest.compId}`,
              },
            },
          } satisfies DevCenterBackofficeModal;
        case 'BACK_OFFICE_PAGE':
          if (manifest.compData.backOfficePage.iframeUrl) {
            return manifest;
          }

          return {
            ...manifest,
            compData: {
              backOfficePage: {
                ...manifest.compData.backOfficePage,
                iframeUrl: `_wix/extensions/backoffice/${manifest.compId}`,
              },
            },
          } satisfies DevCenterBackofficePage;
        case 'ECOM_ADDITIONAL_FEES':
          if (manifest.compData.ecomAdditionalFees.deploymentUri) {
            return manifest;
          }

          return {
            ...manifest,
            compData: {
              ecomAdditionalFees: {
                ...manifest.compData.ecomAdditionalFees,
                deploymentUri: `_wix/extensions/service-plugins/${manifest.compId}`,
              },
            },
          } satisfies DevCenterEcomAdditionalFees;
        case 'ECOM_DISCOUNTS_TRIGGER':
          if (manifest.compData.ecomDiscountsTrigger.deploymentUri) {
            return manifest;
          }

          return {
            ...manifest,
            compData: {
              ecomDiscountsTrigger: {
                ...manifest.compData.ecomDiscountsTrigger,
                deploymentUri: `_wix/extensions/service-plugins/${manifest.compId}`,
              },
            },
          } satisfies DevCenterEcomDiscountsTrigger;
        case 'ECOM_PAYMENT_SETTINGS':
          if (manifest.compData.ecomPaymentSettings.deploymentUri) {
            return manifest;
          }

          return {
            ...manifest,
            compData: {
              ecomPaymentSettings: {
                ...manifest.compData.ecomPaymentSettings,
                deploymentUri: `_wix/extensions/service-plugins/${manifest.compId}`,
              },
            },
          } satisfies DevCenterEcomPaymentSettings;
        case 'ECOM_SHIPPING_RATES':
          if (manifest.compData.ecomShippingRates.deploymentUri) {
            return manifest;
          }

          return {
            ...manifest,
            compData: {
              ecomShippingRates: {
                ...manifest.compData.ecomShippingRates,
                deploymentUri: `_wix/extensions/service-plugins/${manifest.compId}`,
              },
            },
          } satisfies DevCenterEcomShippingRates;
        case 'ECOM_VALIDATIONS':
          if (manifest.compData.ecomValidations.deploymentUri) {
            return manifest;
          }

          return {
            ...manifest,
            compData: {
              ecomValidations: {
                ...manifest.compData.ecomValidations,
                deploymentUri: `_wix/extensions/service-plugins/${manifest.compId}`,
              },
            },
          } satisfies DevCenterEcomValidations;
        case 'GIFT_CARDS_PROVIDER':
          if (manifest.compData.giftCardsProvider.deploymentUri) {
            return manifest;
          }

          return {
            ...manifest,
            compData: {
              giftCardsProvider: {
                ...manifest.compData.giftCardsProvider,
                deploymentUri: `_wix/extensions/service-plugins/${manifest.compId}`,
              },
            },
          } satisfies DevCenterEcomGiftCardsProvider;
        case 'WEBHOOK':
          if (manifest.compData.webhook.callbackUrl) {
            return manifest;
          }

          return {
            ...manifest,
            compData: {
              webhook: {
                ...manifest.compData.webhook,
                callbackUrl: `_wix/extensions/webhooks/${manifest.compId}`,
              },
            },
          } satisfies DevCenterWebhook;
        default:
          return manifest;
      }
    });

  const unknownExtensions = model.unknownComponents.map(
    (component) => component.manifest
  );

  return {
    appId: model.appId,
    components: [...extensions, ...unknownExtensions],
  } satisfies DevCenterAppManifest;
}
