import type { AstroIntegrationLogger } from 'astro';
import chalk from 'chalk';
import { outdent } from 'outdent';
import { loadEnv } from 'vite';

const wixClientIdEnvVar = 'WIX_CLIENT_ID';

export function loadEnvVars(rootDir: string, logger: AstroIntegrationLogger) {
  const env = loadEnv(process.env.NODE_ENV ?? 'development', process.cwd(), '');
  const appId = env[wixClientIdEnvVar];

  if (!appId) {
    logger.error(
      outdent`
        Missing environment variable ${chalk.blueBright(wixClientIdEnvVar)}
        To use the Wix SDK, you must provide the ${chalk.blueBright(
          wixClientIdEnvVar
        )} environment variable.
        💡 To pull the required environment variables from Wix, run:
          ${chalk.magenta('npx wix edge env --env local pull')}
        🔍 Need Help?
        - Visit our docs: https://dev.wix.com/docs/go-headless
        - Join the community: https://discord.com/channels/1114269395317968906/1288424190969511987
      `
    );

    throw new Error(
      `${chalk.magenta(
        wixClientIdEnvVar
      )} not found in loaded environment variables`
    );
  }

  return {
    appId,
    env,
  };
}
