import { fileURLToPath } from 'node:url';
import type { AstroConfig } from 'astro';
import { pathExists } from './fs-utils.js';

interface BuildMetadata {
  appManifestPath: string;
  clientDir: string;
  outDir: string;
  serverDir?: string;
}

export async function resolveBuildMetadata(
  appManifestPath: string,
  config: AstroConfig
): Promise<BuildMetadata> {
  const outDir = fileURLToPath(config.outDir);
  const buildClientDir = fileURLToPath(config.build.client);
  const buildServerDir = fileURLToPath(config.build.server);

  // When there are only static assets, everything is under the out dir.
  // When there are static and server, the output is in the appropriate folders
  const clientDir = (await pathExists(buildClientDir))
    ? buildClientDir
    : outDir;

  // In SSG (static site generation) mode, there is no `serverDir`, only the client side output.
  const serverDir = (await pathExists(buildServerDir))
    ? buildServerDir
    : undefined;

  return {
    appManifestPath,
    clientDir,
    outDir,
    serverDir,
  };
}
