import { rm, writeFile } from 'node:fs/promises';
import { join, resolve } from 'node:path';
import { globby } from 'globby';
import { outdent } from 'outdent';
import type { Model } from '../types.js';
import { isValidWebhookComponent } from './isValidWebhookComponent.js';

export async function writeVirtualWebhookExtensionFiles(
  model: Model,
  codegenDir: string
) {
  const webhookComponents = model.components.filter(isValidWebhookComponent);

  const existingFiles = await globby(`*.ts`, {
    cwd: codegenDir,
    onlyFiles: true,
  });

  for (const filename of existingFiles) {
    const hasMatchingSourceFile = webhookComponents.some(
      (component) => `${component.manifest.compId}.ts` === filename
    );

    if (hasMatchingSourceFile) {
      continue;
    }

    await rm(join(codegenDir, filename));
  }

  for (const component of webhookComponents) {
    const originalEntrypoint = resolve(component.directory, 'event.ts');
    const virtualEntrypoint = join(
      codegenDir,
      `${component.manifest.compId}.ts`
    );

    await writeFile(
      virtualEntrypoint,
      outdent`
        import type { APIRoute } from 'astro';
        import { AppStrategy, createClient } from '@wix/sdk';
        import { WIX_CLIENT_ID } from 'astro:env/client';
        import { WIX_CLIENT_PUBLIC_KEY, WIX_CLIENT_SECRET } from 'astro:env/server';

        const client = createClient({
          auth: AppStrategy({
            appId: WIX_CLIENT_ID,
            appSecret: WIX_CLIENT_SECRET,
            publicKey: WIX_CLIENT_PUBLIC_KEY,
          }),
        });

        const olderClient = globalThis.__wix_context__.client;

        client.enableContext('global');

        await import('${originalEntrypoint}');

        if (olderClient) {
          olderClient.enableContext('global');
        }

        export const ALL: APIRoute = async (context) => {
          await client.webhooks.processRequest(context.request);
          return new Response('OK');
        };
      `
    );
  }
}
