import isCI from 'is-ci';
import { outdent } from 'outdent';
import { defineConfig } from 'tsup';

const viteExternals = ['vite'];
const astroExternals = ['astro', /^astro:/];

export default defineConfig([
  // astro integration
  {
    entry: ['src/index.ts'],
    target: 'node20.9',
    format: ['esm'],
    outDir: 'build',
    splitting: false,
    clean: true,
    sourcemap: true,
    external: [...astroExternals, ...viteExternals],
    dts: {
      resolve: [...astroExternals, ...viteExternals],
    },
    // We specifically do not minify the code in packages **for node** in order to keep it readable for debugging purposes.
    minify: false,
    env: {
      NODE_ENV: isCI ? 'production' : 'development',
    },
    shims: true,
    // https://github.com/evanw/esbuild/issues/1921
    banner: {
      js: outdent`
        import { createRequire as _createRequire } from 'node:module';
        const require = _createRequire(import.meta.url);
      `,
    },
  },
  // backend runtime
  {
    entry: [
      'src/middleware/auth.ts',
      'src/routes/webhooks.ts',
      'src/routes/service-plugins.ts',
    ],
    target: 'node20.9',
    format: ['esm'],
    outDir: 'build-runtime',
    splitting: false,
    external: [...astroExternals, ...viteExternals],
    dts: {
      resolve: [...astroExternals, ...viteExternals],
    },
  },
  // browser runtime
  {
    entry: ['src/context/before-hydration.ts', 'src/context/page.ts'],
    target: 'chrome130',
    format: ['esm'],
    outDir: 'build-browser-runtime',
    splitting: true,
    external: [...astroExternals, ...viteExternals],
    dts: {
      resolve: [...astroExternals, ...viteExternals],
    },
  },
]);
